#include "stdafx.h"
#include "includes.h"

Shader::Shader()
{
	// Zmienne na potrzeby sprawdzenia poprawnosci kompilacji i linkowania:
	GLint success = 0;
	int logLength = 512;
	char* log = new char[logLength];

#pragma region Vertex Shader

	// Kod vertex shadera:
	const char* vsSource = R"(
		#version 330
		
		in vec3 vPosition;

		out vec3 fWorldPosition;

		uniform mat4 modelview;
		uniform mat4 projection;
		
		void main()
		{
			mat4 mvp = projection * modelview;
			gl_Position = mvp * vec4(vPosition, 1.0);

			fWorldPosition = vPosition;
		}
	)";

	GLuint vs = glCreateShader(GL_VERTEX_SHADER); // Pozyskanie id nowego shadera
	glShaderSource(vs, 1, &vsSource, NULL); // Wyslanie kodu zrodlowego
	glCompileShader(vs); // Kompilacja shadera

	// Sprawdzenie poprawnosci kompilacji:
	glGetShaderiv(vs, GL_COMPILE_STATUS, &success);
	if (!success)
	{
		glGetShaderInfoLog(vs, logLength, NULL, log);
		std::cout << "Vertex Shader compilation error:" << std::endl << log;
		exit(EXIT_FAILURE);
	}

#pragma endregion

#pragma region Fragment Shader

	// Kod fragment shadera:
	const char* fsSource = R"(
		#version 330

		in vec3 fWorldPosition;

		out vec4 fragColor;
		
		void main()
		{
			fragColor = vec4(1.0);
		}
	)";

	GLuint fs = glCreateShader(GL_FRAGMENT_SHADER);
	glShaderSource(fs, 1, &fsSource, NULL);
	glCompileShader(fs);

	// Sprawdzenie poprawnosci kompilacji:
	glGetShaderiv(fs, GL_COMPILE_STATUS, &success);
	if (!success)
	{
		glGetShaderInfoLog(fs, logLength, NULL, log);
		std::cout << "Fragment Shader compilation error:" << std::endl << log;
		exit(EXIT_FAILURE);
	}

#pragma endregion

#pragma region Program

	ProgramId = glCreateProgram();
	glAttachShader(ProgramId, vs);
	glAttachShader(ProgramId, fs);
	glLinkProgram(ProgramId);

	// Sprawdzenie poprawnosci linkowania:
	glGetProgramiv(ProgramId, GL_LINK_STATUS, &success);
	if (!success)
	{
		glGetProgramInfoLog(ProgramId, logLength, NULL, log);
		std::cout << "Shading program linking error:" << std::endl << log;
		exit(EXIT_FAILURE);
	}

	// Pozyskanie identyfikatorow zmiennych uniform:
	UniformLoc_modelview = glGetUniformLocation(ProgramId, "modelview");
	UniformLoc_projection = glGetUniformLocation(ProgramId, "projection");

	// Pozyskanie identyfikatorow atrybutow wierzcholkow:
	AttribLoc_vPosition = glGetAttribLocation(ProgramId, "vPosition");

#pragma endregion
}

Shader::~Shader()
{
}
